class Distributions {
    static var Uniform = {
        name: "uniform",
        probability: uniform,
        cumulative: uniform_cdf
    };
    static var Normal = {
        name: "normal",
        probability: normal,
        cumulative: normal_cdf
    };
    static var Bimodal = {
        name: "bimodal",
        probability: bimodal,
        cumulative: bimodal_cdf
    };
    static var sigma = 1;
    static var b0 = 0.2316419;
    static var b1 = 0.319381530;
    static var b2 = -0.356563782;
    static var b3 = 1.781477937;
    static var b4 = -1.821255978;
    static var b5 = 1.330274429;

    static function uniform(x) {
        if (x < -1.5 || x > 1.5) {
            return 0;
        }
        return 1.0/3;
    }

    static function uniform_cdf(x) {
        if (x < -1.5) {
            return 0;
        }
        if (x > 1.5) {
            return 1;
        }
        return (x + 1.5)/3.0;
    }

    static function normal_sigma(x, sigma) {
        var sigma2 = sigma*sigma;
        return Math.exp(-x*x/2/sigma2)/Math.sqrt(2*Math.PI*sigma2);
    }

    static function normal_cdf_sigma(x, sigma) {
        if (x < 0) {
            return 1 - normal_cdf_sigma(-x, sigma);
        }
        x = x / sigma;
        var t = 1/(1 + b0*x);
        var t2 = t*t;
        var t3 = t2*t;
        var t4 = t2*t2;
        var t5 = t4*t;
        return 1 - normal_sigma(x, 1)*(b1*t + b2*t2 + b3*t3 + b4*t4 + b5*t5);
    }

    static function normal(x) {
        return normal_sigma(x, sigma);
    }
    
    static function normal_cdf(x) {
        return normal_cdf_sigma(x, sigma);
    }

    static function normal_cdf0(x) {
        var sign = 1;
        if (x < 0) {
            x = -x;
            sign = -1;
        }
        return 0.5 + sign * 0.5 * Math.sqrt(
            1 - (7*Math.exp(-x*x/2) +
                 16*Math.exp(-x*x*(2-Math.sqrt(2))) +
                (7 + 0.25*Math.PI*x*x)*Math.exp(-x*x))/30);
    }

    static function normal_cdf1(x) {
        var sign = 1;
        if (x < 0) {
            x = -x;
            sign = -1;
        }
        if (x < 2.2) {
            return 0.5 + sign * 0.1 * x * (4.4 - x);
        }
        if (x < 2.6) {
            return 0.5 + sign * 0.49;
        }
        return 0.5 + sign * 0.5;
    }
    
    static function bimodal(x) {
        return (normal_sigma(x - 1, 0.5) + normal_sigma(x + 1, 0.5))/2;
    }
    
    static function bimodal_cdf(x) {
        return (normal_cdf_sigma(x - 1, 0.5) + normal_cdf_sigma(x + 1, 0.5))/2;
    }
}
