class RGB {
    static var white = 0xffffff;
    static var black = 0x000000;
    static var red = 0xff0000;
    static var orange = 0xff8000;
    static var yellow = 0xffff00;
    static var green = 0x00ff00;
    static var cyan = 0x00ffff;
    static var blue = 0x0000ff;
    static var magenta = 0xff00ff;
    
    static function rgb(r, g, b) {
        r = Math.round(r*255);
        g = Math.round(g*255);
        b = Math.round(b*255);
        r = (r < 0) ? 0 : (r > 255) ? 255 : r;
        g = (g < 0) ? 0 : (g > 255) ? 255 : g;
        b = (b < 0) ? 0 : (b > 255) ? 255 : b;
        return (r<<16) + (g<<8) + b;
    }
    
    static function grey(level) {
        return rgb(level, level, level);
    }
    
    static function components(c) {
        var r = (c>>16);
        var g = (c>>8) & 0xff;
        var b = c & 0xff;
        return [r/255.0, g/255.0, b/255.0];
    }
    
    static function blend(c, d, f) {
        c = components(c);
        d = components(d);
        if (f === undefined) f = 0.5;
        var e = 1 - f;
        return rgb(c[0]*e + d[0]*f, c[1]*e + d[1]*f, c[2]*e + d[2]*f);
    }
    
    static function multiply(c, f) {
        c = components(c);
        return rgb(c[0]*f, c[1]*f, c[2]*f);
    }
}
