class VoterPanel extends Element {
    var width, height, range, y_scale, distribution;
    var dot, dot_color, plot, mask;
    var pending_update, moved;

    function VoterPanel(parent, range, width, height, distribution) {
        init(parent);
        this.width = width;
        this.height = height;
        this.y_scale = 1.5;
        this.range = range;

        dot = addclip();
        dot.lineStyle(2, RGB.black, 100);
        dot.beginFill(RGB.white);
        draw_circle(dot, 7, 0, 0);
        dot.endFill(RGB.white);
        dot._x = width/2;
        dot._y = height + 29;
        dot_color = new Color(dot);
        
        mask = new Element(parent);
        mask.clip.beginFill(RGB.black);
        mask.clip.moveTo(0, 0);
        mask.clip.lineTo(width, 0);
        mask.clip.lineTo(width, height);
        mask.clip.lineTo(0, height);
        mask.clip.endFill();

        plot = addclip();
        plot.setMask(mask.clip);
        set_distribution(distribution);

        var voter_panel = this;
        clip.onPress = function() {
            this.startDrag(false, -width/2, -15, width/2, -15);
            voter_panel.dragging = true;
            voter_panel.moved = false;
        }
        clip.onMouseMove = function() {
            if (voter_panel.dragging) {
                voter_panel.pending_update = true;
                voter_panel.moved = true;
            }
        }
        clip.onRelease = function() {
            this.stopDrag();
            voter_panel.dragging = false;
            if (!voter_panel.moved) {
                parent.change_distribution();
            }
        }
        clip.onReleaseOutside = function() {
            this.stopDrag();
            voter_panel.dragging = false;
        }
    }

    function set_distribution(distribution) {
        this.distribution = distribution;
        plot.clear();
        draw_plot(distribution);
    }

    function draw_circle(clip, radius, x, y) {
        var angleDelta = Math.PI / 4;
        var ctrlDist = radius/Math.cos(angleDelta/2);
        var angle = 0;
        var rx, ry, ax, ay;
        clip.moveTo(x + radius, y);
        for (var i = 0; i < 8; i++) {
            angle += angleDelta;
            rx = x + Math.cos(angle-(angleDelta/2))*(ctrlDist);
            ry = y + Math.sin(angle-(angleDelta/2))*(ctrlDist);
            ax = x + Math.cos(angle)*radius;
            ay = y + Math.sin(angle)*radius;
            clip.curveTo(rx, ry, ax, ay);
        }
    }

    function world(x) {
        return (x*2/width - 1)*range;
    }
    
    function screen(position) {
        return ((position/range) + 1)*width/2;
    }
        
    function draw_plot(distribution) {
        var pf = distribution.probability;
        plot.lineStyle(0, 0, 0);
        plot.beginFill(RGB.grey(0.5));
        plot.moveTo(-width, height);
        plot.lineTo(-width, height * (1 - pf(world(-width))));
        for (var x = -width; x <= width*2; x++) {
            var y = height * (1 - pf(world(x))*y_scale);
            plot.lineTo(x, y);
        }
        plot.lineTo(width*2, height);
        plot.endFill();
    }

    function get_x() {
        return clip._x + width/2;
    }

    function move_to(x, row) {
        clip._x = x - width/2;
        dot._y = height + 29 + row*(28+15);
    }

    function show_election(color) {
        var rgb = RGB.components(color || RGB.grey(0.6));
        dot_color.setTransform(
            {ra: 100*rgb[0], ga: 100*rgb[1], ba: 100*rgb[2]});
    }
}
